package com.muyu.etl.util.factory;

import com.alibaba.druid.pool.DruidDataSource;
import com.alibaba.druid.pool.DruidPooledConnection;
import com.muyu.etl.domain.BasicConfigInfo;
import com.muyu.etl.util.service.ConnectionPoolFactory;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.HashMap;

public class DruidUtilsFactory implements ConnectionPoolFactory<BasicConfigInfo> {
    // 连接池容器
    public static HashMap<String, DruidPooledConnection> map = new HashMap<>();

    /**
     * 初始化连接池
     *
     * @param basicConfigInfo
     * @return
     * @throws SQLException
     */
    @Override
    public DruidPooledConnection init(BasicConfigInfo basicConfigInfo) throws SQLException {
        // 连接池对象
        DruidDataSource source = new DruidDataSource();
        //  定义下面需要的对象
        String host = basicConfigInfo.getHost();
        String port = basicConfigInfo.getPort();
        String databaseName = basicConfigInfo.getDatabaseName();
        String databaseType = basicConfigInfo.getDatabaseType();
        // 引擎配置
        source.setUrl("jdbc:" + databaseType + "://" + host + ":" + port + "/" + databaseName + "?" + basicConfigInfo.getConnectionParams());
        source.setUsername(basicConfigInfo.getUsername());
        source.setPassword(basicConfigInfo.getPassword());
        //初始化连接量
        source.setInitialSize(Math.toIntExact(basicConfigInfo.getInitLinkNum()));
        //最大连接数
        source.setMaxActive(Math.toIntExact(basicConfigInfo.getMaxLinkNum()));
        source.setMaxWaitThreadCount(Math.toIntExact(basicConfigInfo.getMaxWaitTimes()));
        //连接池中的最大最小生存时间
        source.setMaxEvictableIdleTimeMillis(basicConfigInfo.getMaxWaitTime());
        source.setMinEvictableIdleTimeMillis(30000);
        //最大等待时间
        source.setTimeBetweenEvictionRunsMillis(30000);
        //进程最大等待时间
        source.setMaxWait(basicConfigInfo.getMaxWaitTime());
        DruidPooledConnection pool = source.getConnection();
        map.put(host + ":" + port + "/" + databaseName, pool);
        //  获取并返回连接池对象
        return pool;
    }

    /**
     * 根据接入取出一个连接对象
     *
     * @param basicConfigInfo
     * @return
     */
    @Override
    public Connection getConnection(BasicConfigInfo basicConfigInfo) {
        String host = basicConfigInfo.getHost();
        String port = basicConfigInfo.getPort();
        String databaseName = basicConfigInfo.getDatabaseName();
        DruidPooledConnection druidPooledConnection = map.get(host + ":" + port + "/" + databaseName);
        Connection connection = druidPooledConnection.getConnection();
        return connection;
    }

    /**
     * 将连接对象放回连接池
     *
     * @param basicConfigInfo
     * @param connection
     */
    @Override
    public void giveBack(BasicConfigInfo basicConfigInfo, Connection connection) throws SQLException {
        String host = basicConfigInfo.getHost();
        String port = basicConfigInfo.getPort();
        String databaseName = basicConfigInfo.getDatabaseName();
        DruidPooledConnection druidPooledConnection = map.get(host + ":" + port + "/" + databaseName);
        druidPooledConnection.close();
    }


}
