package com.muyu.engine.service.impl;

import java.io.*;
import java.lang.reflect.Method;
import java.util.List;

import com.alibaba.nacos.shaded.com.google.common.base.Supplier;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.conditions.update.LambdaUpdateChainWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.utils.DateUtils;
import com.muyu.engine.domain.RuleEngine;
import com.muyu.engine.service.RuleEngineService;
import io.lettuce.core.dynamic.support.ReflectionUtils;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.muyu.engine.mapper.RuleEngineVersionMapper;
import com.muyu.engine.domain.RuleEngineVersion;
import com.muyu.engine.service.RuleEngineVersionService;

import javax.servlet.ServletException;

/**
 * 规则版本Service业务层处理
 *
 * @author Saisai
 * @date 2024-05-06
 */
@Service
@Log4j2
public class RuleEngineVersionServiceImpl extends ServiceImpl<RuleEngineVersionMapper, RuleEngineVersion> implements RuleEngineVersionService {
    @Autowired
    private RuleEngineVersionMapper ruleEngineVersionMapper;
    @Autowired
    private RuleEngineService ruleEngineService;

    /**
     * 查询规则版本
     *
     * @param id 规则版本主键
     * @return 规则版本
     */
    @Override
    public RuleEngineVersion selectRuleEngineVersionById(Long id) {
        return ruleEngineVersionMapper.selectRuleEngineVersionById(id);
    }

    /**
     * 查询规则版本列表
     *
     * @param ruleEngineId 规则版本
     * @return 规则版本
     */
    @Override
    public List<RuleEngineVersion> selectRuleEngineVersionList(Long ruleEngineId) {
        return list(new LambdaQueryWrapper<RuleEngineVersion>() {{
            eq(RuleEngineVersion::getRuleEngineId, ruleEngineId);
        }});
    }

    /**
     * 新增规则版本
     *
     * @param ruleEngineVersion 规则版本
     * @return 结果
     */
    @Override
    public int insertRuleEngineVersion(RuleEngineVersion ruleEngineVersion) {
        ruleEngineVersion.setCreateTime(DateUtils.getNowDate());
        return ruleEngineVersionMapper.insertRuleEngineVersion(ruleEngineVersion);
    }

    /**
     * 修改规则版本
     *
     * @param ruleEngineVersion 规则版本
     * @return 结果
     */
    @Override
    public int updateRuleEngineVersion(RuleEngineVersion ruleEngineVersion) {
        ruleEngineVersion.setUpdateTime(DateUtils.getNowDate());
        return ruleEngineVersionMapper.updateRuleEngineVersion(ruleEngineVersion);
    }

    /**
     * 批量删除规则版本
     *
     * @param ids 需要删除的规则版本主键
     * @return 结果
     */
    @Override
    public int deleteRuleEngineVersionByIds(Long[] ids) {
        return ruleEngineVersionMapper.deleteRuleEngineVersionByIds(ids);
    }

    /**
     * 删除规则版本信息
     *
     * @param id 规则版本主键
     * @return 结果
     */
    @Override
    public int deleteRuleEngineVersionById(Long id) {
        return ruleEngineVersionMapper.deleteRuleEngineVersionById(id);
    }

    /**
     * @param ruleEngineVersion
     * @return
     */
    @Override
    public boolean saveCoding(RuleEngineVersion ruleEngineVersion) throws ServletException {
        RuleEngineVersion one = this.getOne(new LambdaQueryWrapper<>() {{
            eq(RuleEngineVersion::getCode, ruleEngineVersion.getCode());
        }});
        if (!"null".equals(one)) throw new ServletException("该版本已存在");
        String codeIng = ruleEngineVersion.getCodeIng();
        try {
            String path = codeIng.substring(codeIng.indexOf("com"), codeIng.indexOf(";")).replaceAll("\\.", "/").trim();
            String fileName = codeIng.substring(codeIng.indexOf("class") + 6, codeIng.indexOf("{")).trim();
            //去除父级
            if (fileName.contains("extends")) {
                fileName = fileName.substring(0, fileName.indexOf("extends")).trim();
            }
            //去除实现接口
            if (fileName.contains("implements")) {
                fileName = fileName.substring(0, fileName.indexOf("implements")).trim();
            }
            //代码编译
            File file = new File("D:/ruoyi/FinallyTest/muyu-modules/muyu-ruleEngine/muyu-ruleEngine-redis/src/main/java/" + path + "/" + fileName + ".java");
            if (!file.exists()) {
                //不存在创建
                file.createNewFile();
            } else {
                //存在测重新创建
                file.delete();
                file.createNewFile();
            }
            String[] split = codeIng.split("/n");
            //OutputStreamWriter对象将字符转换为字节流，指定了文件输出流和编码方式。
            OutputStreamWriter outputStreamWriter =
                    new OutputStreamWriter(new FileOutputStream(file, true), "UTF-8");
            //BufferedWriter对象，用于缓冲写入数据，提高写入效率。
            BufferedWriter bw = new BufferedWriter(outputStreamWriter);
            for (String str : split) {
                bw.write(str);
                bw.newLine();
            }
            bw.close();
            outputStreamWriter.close();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        log.info("初始化成功");
        ruleEngineVersion.setStatus("1");
        boolean update = this.update(ruleEngineVersion, new LambdaUpdateWrapper<RuleEngineVersion>() {{
            eq(RuleEngineVersion::getId, ruleEngineVersion.getId());
        }});
        return update;

    }


    /**
     * 测试引擎版本
     *
     * @param
     * @return
     * @throws Exception
     */
    @Override
    public boolean testEngine(String codeIng) throws Exception {
        String path = codeIng.substring(codeIng.indexOf("com"), codeIng.indexOf(";")).replaceAll("/.", "/").trim();
        String fileName = codeIng.substring(codeIng.indexOf("class") + 6, codeIng.indexOf("{")).trim();
        String name = path + "." + fileName;
        String javaPackageName = name.replace(".", File.separator) + ".java";
        String javaAbsolutePath = "D:/ruoyi/FinallyTest/muyu-modules/muyu-ruleEngine/muyu-ruleEngine-redis/src/main/java/" + javaPackageName;
        String jarAbsolutePath = "D:/ruoyi/FinallyTest/muyu-modules/muyu-ruleEngine/muyu-ruleEngine-redis/target/classes/com/muyu/engine/domain/test";
        Process process = Runtime.getRuntime().exec("javac -classpath " + jarAbsolutePath + " " + javaAbsolutePath);
        try {
            int exitVal = process.waitFor();
            System.out.println("Process exitValue: " + exitVal);
            ClassLoader classLoader = ReflectionUtils.class.getClassLoader();
            Class aClass = classLoader.loadClass(name);
            Object o = aClass.newInstance();
            Method[] methods = aClass.getMethods();
            for (Method method : methods) {
                method.invoke(o, null);
            }
        } catch (Exception e) {
            log.error(e.getMessage());
        }
        return true;
    }

}
