package com.muyu.custom;

import com.alibaba.druid.pool.DruidPooledConnection;
import com.alibaba.fastjson2.JSON;
import com.muyu.etl.RemoteAssetService;
import com.muyu.etl.domain.BasicConfigInfo;
import com.muyu.etl.util.service.ConnectionPoolFactory;
import com.rabbitmq.client.Channel;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.servlet.ServletException;
import java.io.IOException;
import java.sql.SQLException;

/**
 * 初始化添加的连接池接入配置
 * @ClassName AddInitConn
 * @Description 初始化添加的连接池接入配置
 * @Author SaiSai.Liu
 * @Date 2024/5/12 14:18
 */

@Component
@Log4j2
public class AddInitConn {
    @Autowired
    private RabbitTemplate rabbitTemplate;
    @Autowired
    private ConnectionPoolFactory<BasicConfigInfo> connectionPoolFactory;
    @RabbitListener(queuesToDeclare = {@Queue(name = "basicInfoQueue")})
    public void myConsumer(String basicInfo, Message message, Channel channel) throws ServletException {
        if (basicInfo == null){
            throw new ServletException("队列内容为空");
        }
        BasicConfigInfo basicConfigInfo = JSON.parseObject(basicInfo, BasicConfigInfo.class);
        try {
            DruidPooledConnection init = connectionPoolFactory.init(basicConfigInfo);
            if (init == null){
                // 消息拒绝，放回队列中重新消费
                channel.basicReject(message.getMessageProperties().getDeliveryTag(), true);
            }else {
                // 消息消费确认
                channel.basicAck(message.getMessageProperties().getDeliveryTag(), true);
            }
        } catch (SQLException | IOException e) {
            log.error("初始化连接池失败，消费失败");
            throw new RuntimeException(e);
        }
    }
}
